<table width=100%>
	   <tr>
	   	   <td width=20%><a href="http://www.ligo.caltech.edu">
     	   	   <img src="./img/LIGOLogo.gif" border="0"></a></td>
		   <td><center><h2><font face="arial"><?php echo $uTitle ?></font></center></h2></td>
		   <td width=20%><right><a href="http://www.ligo.org">
		       <img align="right" src="./img/lsc.gif" border="0"></right></td>
	   </tr>
</table>
<hr width="100%">
